/*
 * Copyright Aduna (http://www.aduna-software.com/) (c) 2007.
 *
 * Licensed under the Aduna BSD-style license.
 */
package info.aduna.app.logging.logback;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import ch.qos.logback.classic.LoggerContext;
import ch.qos.logback.core.joran.spi.JoranException;
import ch.qos.logback.core.util.StatusPrinter;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import info.aduna.app.logging.base.LogConfigurationBase;
import info.aduna.io.IOUtil;
import info.aduna.io.ResourceUtil;
import info.aduna.logging.LogReader;
import info.aduna.logging.file.logback.FileLogReader;

public class LogbackConfiguration extends LogConfigurationBase {

	private final Logger logger = LoggerFactory.getLogger(this.getClass());

	private static final String LOGBACK_CONFIG_FILE = "logback.xml";

	private static final String DEFAULT_LOGBACK_CONFIG_FILE = "default/" + LOGBACK_CONFIG_FILE;

	private File configFile;

	private File logFile;
	
	private File userAccessLogFile;
	
	private File adminAccessLogFile;

	private LogConfigurator configurator = null;

	public LogbackConfiguration()
		throws IOException
	{
		super();
		// USE init() FOR FURTHER CONFIGURATION
		// it will be called from the super constructor
	}

	public void init()
		throws IOException
	{
		logFile = new File(getLoggingDir(), LOGGING_FILE);
		userAccessLogFile = new File(getLoggingDir(), USER_ACCESS_LOGGING_FILE);
		adminAccessLogFile = new File(getLoggingDir(), ADMIN_ACCESS_LOGGING_FILE);

		configFile = getConfigFile();

		load();

		logger.info("Logback logging API implementation is configured.");
		logger.debug("Log file: " + logFile.getAbsolutePath());

		save();
	}

	public void load()
		throws IOException
	{
		LoggerContext lc = (LoggerContext)LoggerFactory.getILoggerFactory();
		try {
			configurator = new LogConfigurator();
			configurator.setContext(lc);
			lc.shutdownAndReset();
			configurator.doConfigure(configFile);
		}
		catch (JoranException je) {
			System.out.println("Logback configuration error");
			je.printStackTrace();
			StatusPrinter.print(lc);
		}
	}

	public void save()
		throws IOException
	{
		// nop
	}

	public void destroy() {
		// look up all loggers in the logger context and close
		// all appenders configured for them.
		LoggerContext lc = (LoggerContext)LoggerFactory.getILoggerFactory();
		lc.shutdownAndReset();
	}

	private File getConfigFile()
		throws IOException
	{
		File f = new File(getConfDir(), LOGBACK_CONFIG_FILE);
		if (!f.canRead()) {
			URL url = ResourceUtil.getURL(DEFAULT_LOGBACK_CONFIG_FILE);
			if (url == null) {
				throw new RuntimeException("No required resource " + DEFAULT_LOGBACK_CONFIG_FILE + " is found!");
			}

			String content = IOUtil.readString(url);
			content = content.replace("${logging.Mainlog.file}", logFile.getAbsolutePath());
			content = content.replace("${logging.useraccess.file}", userAccessLogFile.getAbsolutePath());
			content = content.replace("${logging.adminaccess.file}", adminAccessLogFile.getAbsolutePath());
			content = content.replace("${logging.useraccess.logger}", USER_ACCESS_LOGGER_NAME);
			content = content.replace("${logging.adminaccess.logger}", ADMIN_ACCESS_LOGGER_NAME);
			if (!f.getParentFile().mkdirs() && !f.getParentFile().canWrite()) {
				throw new IOException("Not allowed to write logging configuration file to " + f.getParent());
			}
			else {
				IOUtil.writeString(content, f);
			}
		}
		return f;
	}

	public LogReader getLogReader(String appender) {
		return this.configurator.getLogReader(appender);
	}

	public LogReader getDefaultLogReader() {
		LogReader logReader = this.configurator.getDefaultLogReader();
		if (logReader != null) {
			return logReader;
		}
		return new FileLogReader(this.logFile);
	}

}
