/*
 * Copyright Aduna (http://www.aduna-software.com/) (c) 1997-2007.
 *
 * Licensed under the Aduna BSD-style license.
 */

package org.slf4j.impl;

import org.slf4j.helpers.MarkerIgnoringBase;
import org.slf4j.helpers.MessageFormatter;

public class SimpleConfigurableLogger extends MarkerIgnoringBase {

	private static final SimpleConfigurableLogger SINGLETON = new SimpleConfigurableLogger();

	/**
	 * Returns the global SimpleConfigurableLogger object.
	 */
	public static SimpleConfigurableLogger getLogger() {
		return SINGLETON;
	}

	/**
	 * Mark the time when this class gets loaded into memory.
	 */
	private static long startTime = System.currentTimeMillis();

	private static String TRACE_STR = "TRACE";

	private static String DEBUG_STR = "DEBUG";

	private static String INFO_STR = "INFO";

	private static String WARN_STR = "WARN";

	private static String ERROR_STR = "ERROR";

	private boolean traceEnabled = true;

	private boolean debugEnabled = true;

	private boolean infoEnabled = true;

	private boolean warnEnabled = true;

	private boolean errorEnabled = true;

	/**
	 * Private constructor to enforce use of {@link #getLogger()}.
	 */
	private SimpleConfigurableLogger() {
		// nop
	}

	public String getName() {
		return "single instance";
	}

	/**
	 * This is our internal implementation for logging regular
	 * (non-parameterized) log messages.
	 *
	 * @param level
	 * @param message
	 * @param t
	 */
	private void log(String level, String message, Throwable t) {
		StringBuilder result = new StringBuilder(t == null ? 256 : 2048);

		long millis = System.currentTimeMillis();
		result.append(millis - startTime);

		result.append(" [");
		result.append(Thread.currentThread().getName());
		result.append("] ");

		result.append(level);
		result.append(" - ");

		result.append(message);

		System.err.println(result.toString());

		if (t != null) {
			t.printStackTrace(System.err);
		}

		System.err.flush();
	}

	/**
	 * For formatted messages, first substitute arguments and then log.
	 *
	 * @param level
	 * @param format
	 * @param param1
	 * @param param2
	 */
	private void formatAndLog(String level, String format, Object arg1, Object arg2) {
		String message = MessageFormatter.format(format, arg1, arg2);
		log(level, message, null);
	}

	/**
	 * For formatted messages, first substitute arguments and then log.
	 *
	 * @param level
	 * @param format
	 * @param argArray
	 */
	private void formatAndLog(String level, String format, Object[] argArray) {
		String message = MessageFormatter.arrayFormat(format, argArray);
		log(level, message, null);
	}

	/**
	 * Always returns false.
	 *
	 * @return always false
	 */
	public boolean isTraceEnabled() {
		return traceEnabled;
	}

	public void setTraceEnabled(boolean enabled) {
		traceEnabled = enabled;

		if (enabled) {
			setDebugEnabled(true);
		}
	}

	/**
	 * A NOP implementation, as this logger is permanently disabled for the DEBUG
	 * level.
	 */
	public void trace(String msg) {
		if (isTraceEnabled()) {
			log(TRACE_STR, msg, null);
		}
	}

	/**
	 * A NOP implementation, as this logger is permanently disabled for the DEBUG
	 * level.
	 */
	public void trace(String format, Object arg) {
		if (isTraceEnabled()) {
			formatAndLog(TRACE_STR, format, arg, null);
		}
	}

	/**
	 * A NOP implementation, as this logger is permanently disabled for the DEBUG
	 * level.
	 */
	public void trace(String format, Object arg1, Object arg2) {
		if (isTraceEnabled()) {
			formatAndLog(TRACE_STR, format, arg1, arg2);
		}
	}

	public void trace(String format, Object[] argArray) {
		if (isTraceEnabled()) {
			formatAndLog(TRACE_STR, format, argArray);
		}
	}

	/**
	 * A NOP implementation, as this logger is permanently disabled for the DEBUG
	 * level.
	 */
	public void trace(String msg, Throwable t) {
		if (isTraceEnabled()) {
			log(TRACE_STR, msg, t);
		}
	}


	/**
	 * Always returns false.
	 *
	 * @return always false
	 */
	public boolean isDebugEnabled() {
		return debugEnabled;
	}

	public void setDebugEnabled(boolean enabled) {
		debugEnabled = enabled;

		if (enabled) {
			setInfoEnabled(true);
		}
	}

	/**
	 * A NOP implementation, as this logger is permanently disabled for the DEBUG
	 * level.
	 */
	public void debug(String msg) {
		if (isDebugEnabled()) {
			log(DEBUG_STR, msg, null);
		}
	}

	/**
	 * A NOP implementation, as this logger is permanently disabled for the DEBUG
	 * level.
	 */
	public void debug(String format, Object arg) {
		if (isDebugEnabled()) {
			formatAndLog(DEBUG_STR, format, arg, null);
		}
	}

	/**
	 * A NOP implementation, as this logger is permanently disabled for the DEBUG
	 * level.
	 */
	public void debug(String format, Object arg1, Object arg2) {
		if (isDebugEnabled()) {
			formatAndLog(DEBUG_STR, format, arg1, arg2);
		}
	}

	public void debug(String format, Object[] argArray) {
		if (isDebugEnabled()) {
			formatAndLog(DEBUG_STR, format, argArray);
		}
	}

	/**
	 * A NOP implementation, as this logger is permanently disabled for the DEBUG
	 * level.
	 */
	public void debug(String msg, Throwable t) {
		if (isDebugEnabled()) {
			log(TRACE_STR, msg, t);
		}
	}

	/**
	 * Always returns true.
	 */
	public boolean isInfoEnabled() {
		return infoEnabled;
	}

	public void setInfoEnabled(boolean enabled) {
		infoEnabled = enabled;

		if (enabled) {
			setWarnEnabled(true);
		}
		else {
			setDebugEnabled(false);
		}
	}

	/**
	 * A simple implementation which always logs messages of level INFO according
	 * to the format outlined above.
	 */
	public void info(String msg) {
		if (isInfoEnabled()) {
			log(INFO_STR, msg, null);
		}
	}

	/**
	 * Perform single parameter substitution before logging the message of level
	 * INFO according to the format outlined above.
	 */
	public void info(String format, Object arg) {
		if (isInfoEnabled()) {
			formatAndLog(INFO_STR, format, arg, null);
		}
	}

	/**
	 * Perform double parameter substitution before logging the message of level
	 * INFO according to the format outlined above.
	 */
	public void info(String format, Object arg1, Object arg2) {
		if (isInfoEnabled()) {
			formatAndLog(INFO_STR, format, arg1, arg2);
		}
	}

	/**
	 * Perform double parameter substitution before logging the message of level
	 * INFO according to the format outlined above.
	 */
	public void info(String format, Object[] argArray) {
		if (isInfoEnabled()) {
			formatAndLog(INFO_STR, format, argArray);
		}
	}

	/**
	 * Log a message of level INFO, including an exception.
	 */
	public void info(String msg, Throwable t) {
		if (isInfoEnabled()) {
			log(INFO_STR, msg, t);
		}
	}

	/**
	 * Always returns true.
	 */
	public boolean isWarnEnabled() {
		return warnEnabled;
	}

	public void setWarnEnabled(boolean enabled) {
		warnEnabled = enabled;

		if (enabled) {
			setErrorEnabled(true);
		}
		else {
			setInfoEnabled(false);
		}
	}

	/**
	 * A simple implementation which always logs messages of level WARN according
	 * to the format outlined above.
	 */
	public void warn(String msg) {
		if (isWarnEnabled()) {
			log(WARN_STR, msg, null);
		}
	}

	/**
	 * Perform single parameter substitution before logging the message of level
	 * WARN according to the format outlined above.
	 */
	public void warn(String format, Object arg) {
		if (isWarnEnabled()) {
			formatAndLog(WARN_STR, format, arg, null);
		}
	}

	/**
	 * Perform double parameter substitution before logging the message of level
	 * WARN according to the format outlined above.
	 */
	public void warn(String format, Object arg1, Object arg2) {
		if (isWarnEnabled()) {
			formatAndLog(WARN_STR, format, arg1, arg2);
		}
	}

	/**
	 * Perform double parameter substitution before logging the message of level
	 * WARN according to the format outlined above.
	 */
	public void warn(String format, Object[] argArray) {
		if (isWarnEnabled()) {
			formatAndLog(WARN_STR, format, argArray);
		}
	}

	/**
	 * Log a message of level WARN, including an exception.
	 */
	public void warn(String msg, Throwable t) {
		if (isWarnEnabled()) {

			log(WARN_STR, msg, t);
		}
	}

	/**
	 * Always returns true.
	 */
	public boolean isErrorEnabled() {
		return errorEnabled;
	}

	public void setErrorEnabled(boolean enabled) {
		errorEnabled = enabled;

		if (!enabled) {
			setWarnEnabled(false);
		}
	}

	/**
	 * A simple implementation which always logs messages of level ERROR
	 * according to the format outlined above.
	 */
	public void error(String msg) {
		if (isErrorEnabled()) {
			log(ERROR_STR, msg, null);
		}
	}

	/**
	 * Perform single parameter substitution before logging the message of level
	 * ERROR according to the format outlined above.
	 */
	public void error(String format, Object arg) {
		if (isErrorEnabled()) {
			formatAndLog(ERROR_STR, format, arg, null);
		}
	}

	/**
	 * Perform double parameter substitution before logging the message of level
	 * ERROR according to the format outlined above.
	 */
	public void error(String format, Object arg1, Object arg2) {
		if (isErrorEnabled()) {
			formatAndLog(ERROR_STR, format, arg1, arg2);
		}
	}

	/**
	 * Perform double parameter substitution before logging the message of level
	 * ERROR according to the format outlined above.
	 */
	public void error(String format, Object[] argArray) {
		if (isErrorEnabled()) {
			formatAndLog(ERROR_STR, format, argArray);
		}
	}

	/**
	 * Log a message of level ERROR, including an exception.
	 */
	public void error(String msg, Throwable t) {
		if (isErrorEnabled()) {
			log(ERROR_STR, msg, t);
		}
	}
}
